package com.autentia.demo.ejb;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.net.URL;
import java.util.Arrays;
import java.util.List;

import javax.naming.InitialContext;
import javax.persistence.EntityManager;

import junit.framework.JUnit4TestAdapter;

import org.jboss.deployers.spi.DeploymentException;
import org.jboss.embedded.Bootstrap;
import org.jboss.virtual.plugins.context.vfs.AssembledContextFactory;
import org.jboss.virtual.plugins.context.vfs.AssembledDirectory;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

public class EJBsTest {

	/**
	 * Lista de mensajes de bienvenida que se van a intentar añadir.
	 */
	private static final List<String> GREETING_MESSAGES = Arrays.asList("Hola tronco !!!", "Hola culo !!!",
			"Bienvenido Sr.", "Encantado de verle Sr. culo !!!");

	/**
	 * Simula un jar con todos los elementos que vamos a desplegar en el microcontenedor (Embedded JBoss).
	 */
	private static AssembledDirectory jar;

	/**
	 * Runs the test suite using the textual runner.
	 */
	public static void main(String[] args) {
		junit.textui.TestRunner.run(suite());
	}

	/**
	 * Hace el despliegue de todos los elementos que necesitamos para ejecutar los test. Simula la construcción de un jar, y
	 * como si luego echáramos este jar al directorio server/default/deploy del JBoss.
	 * <p>
	 * <b>¡¡¡ Muy importante no olvidar poner todos los elementos EJBs de sesión, entidad, fichero de configuración de la
	 * persistencia, ... !!!</b>
	 * 
	 * @throws DeploymentException si no se puede hacer el despliegue.
	 */
	private static void deploy() throws DeploymentException {
		jar = AssembledContextFactory.getInstance().create("ejbTestCase.jar");
		jar.addClass(Greeting.class);
		jar.addClass(Greeter.class);
		jar.addClass(DummyGreeterBean.class);
		jar.addClass(SmartGreeterBean.class);
		jar.addClass(GreetingModerator.class);
		jar.addClass(GreetingModeratorBean.class);
		jar.addClass(PoliteSmartGreeterBean.class);
		jar.mkdir("META-INF").addResource("test-persistence.xml", "persistence.xml");
		Bootstrap.getInstance().deploy(jar);
	}

	/**
	 * Elimina los elementos que se habían desplegado con deploy();
	 * 
	 * @throws DeploymentException si no se puede hacer la desinstalación.
	 */
	private static void undeploy() throws DeploymentException {
		Bootstrap.getInstance().undeploy(jar);
		AssembledContextFactory.getInstance().remove(jar);
	}

	/**
	 * Este método determina los métodos que se van a ejecutar con el test. Se usa la clase JUnit4TestAdapter para que se
	 * puedan ejecutar los test desde un entorno JUnit 3.
	 * 
	 * @return el conjutno de test que hay que ejecutar.
	 */
	public static junit.framework.Test suite() {
		final junit.framework.Test suite = new JUnit4TestAdapter(EJBsTest.class);
		return suite;
	}

	/**
	 * Hacer la inicialización del Embedded JBoss pueden ser unos 5 sg. así que nos aseguramos que se haga una única vez
	 * antes de ejecutar todos los test.
	 * 
	 * @throws DeploymentException si no se puede hacer el despliegue de los elementos para ejecutar los tests.
	 */
	@BeforeClass
	public static void setUpBeforeAllTest() throws Exception {
		if (!Bootstrap.getInstance().isStarted()) {

			// IMPORTATNE !!!
			// Estas dos líneas son necesarias para poder ejecutar los test desde Maven.
			// Es culpa de un bug (http://jira.codehaus.org/browse/SUREFIRE-104) de del Surefire (plugin de Maven que lanza
			// los test)
			// Desde ant o Eclipse o ... no serían necesarias, pero tenerlas puestas tampoco no afecta negativamente.
			org.jboss.net.protocol.URLStreamHandlerFactory factory = new org.jboss.net.protocol.URLStreamHandlerFactory();
			URL.setURLStreamHandlerFactory(factory);

			Bootstrap.getInstance().bootstrap();
		}
		deploy();
	}

	/**
	 * Parada del Embedded JBoss. Nos aseguramos que se haga una única vez después de ejecutar todos los test.
	 * 
	 * @throws DeploymentException si no se puede la desinstalación de los elementos para ejecutar los tests.
	 */
	@AfterClass
	public static void tearDownAfterAllTest() throws Exception {
		undeploy();
		if (System.getProperty("shutdown.embedded.jboss") != null) {
			Bootstrap.getInstance().shutdown();
		}
	}

	private String sayHi(String ejbName) throws Exception {
		final InitialContext ctx = new InitialContext();
		final Greeter greeter = (Greeter)ctx.lookup(ejbName);
		return greeter.sayHi();
	}

	private void addGreeting(String ejbName, String message) throws Exception {
		final InitialContext ctx = new InitialContext();
		final Greeter greeter = (Greeter)ctx.lookup(ejbName);
		greeter.addGreeting(message);
	}

	@Test
	public void dummyGreeterBean_sayHi() throws Exception {
		final String greeting = sayHi("DummyGreeterBean/local");
		assertEquals(DummyGreeterBean.DEFAULT_GREETING, greeting);
	}

	@Test
	public void smartGreeterBean_sayHi() throws Exception {
		final String greeting = sayHi("SmartGreeterBean/local");
		assertEquals(Greeting.DEFAULT_MESSAGE, greeting);
	}

	@Test
	public void politeSmartGreeterBean_sayHi() throws Exception {
		final String greeting = sayHi("PoliteSmartGreeterBean/local");
		assertEquals(Greeting.DEFAULT_MESSAGE, greeting);
	}

	@Test
	public void smartGreeterBean_addPoliteGreting() throws Exception {
		addGreeting("SmartGreeterBean/local", GREETING_MESSAGES.get(0));
	}

	@Test
	public void smartGreeterBean_addUnpoliteGreting() throws Exception {
		addGreeting("SmartGreeterBean/local", GREETING_MESSAGES.get(1));
	}

	@Test
	public void politeSmartGreeterBean_addPoliteGreting() throws Exception {
		addGreeting("PoliteSmartGreeterBean/local", GREETING_MESSAGES.get(2));
	}

	/**
	 * Prueba a añadir un mensaje de bienvenida poco educado. El EJB se negará y lanzará una excepción.
	 * Configuramos JUnit para que tenga en cuenta esta situación y de por válido el test sólo si se lanza
	 * la excepción.
	 * 
	 * @throws Exception si no se puede añadir el mensaje de bienvenida.
	 */
	@Test(expected = InvalidGreetingMessageException.class)
	public void politeSmartGreeterBean_addUnpoliteGreting() throws Exception {
		addGreeting("PoliteSmartGreeterBean/local", GREETING_MESSAGES.get(3));
	}

	/**
	 * Comprueba que se han añadido correctamente en la base de datos todos los mensajes de bienvenida.
	 * Estarán todos - 1, el último, que por ser poco educado el EJB se negó a añadirlo.
	 * 
	 * @throws Exception si se produce cualquier error al ejecutar el test.
	 */
	@Test
	public void checkGreetingsInDataBase() throws Exception {
		final EntityManager em = (EntityManager)new InitialContext().lookup("java:/EntityManagers/test-autentia");

		final List<Greeting> greetings = em.createQuery("from Greeting").getResultList();

		int greetingsInDataBase = 0;
		for (Greeting greeting : greetings) {
			assertTrue(GREETING_MESSAGES.contains(greeting.getMessage()));
			greetingsInDataBase++;
		}
		
		// Tiene que haber tantos como el tamaño de la lista - 1 porque uno no es educado
		// y el PoliteSmartGreeterBean se ha tenido que negar a añadirlo.
		assertEquals(GREETING_MESSAGES.size() - 1, greetingsInDataBase);
	}

}
